@@ -26,6 +26,9 @@ from .widgets import (
     VariableMixin,
     MultiRangeBar,
     MultiProgressBar,
+    GranularBar,
+    FormatLabelBar,
+    PercentageLabelBar,
     Variable,
     DynamicMessage,
     FormatCustomText,
@@ -72,6 +75,9 @@ __all__ = [
     'VariableMixin',
     'MultiRangeBar',
     'MultiProgressBar',
+    'GranularBar',
+    'FormatLabelBar',
+    'PercentageLabelBar',
     'Variable',
     'DynamicMessage',
     'FormatCustomText',
