@@ -1,4 +1,6 @@
-from .six import with_metaclass
+# -*- mode: python; coding: utf-8 -*-
+from __future__ import absolute_import
+import six
 
 
 class FalseMeta(type):
@@ -11,5 +13,9 @@ class FalseMeta(type):
     __nonzero__ = __bool__
 
 
-class UnknownLength(with_metaclass(FalseMeta, object)):
+class UnknownLength(six.with_metaclass(FalseMeta, object)):
+    pass
+
+
+class Undefined(six.with_metaclass(FalseMeta, object)):
     pass
