@@ -114,15 +114,19 @@ class FormatWidgetMixin(object):
         self.new_style = new_style
         self.format = format
 
+    def get_format(self, progress, data, format=None):
+        return format or self.format
+
     def __call__(self, progress, data, format=None):
         '''Formats the widget into a string'''
+        format = self.get_format(progress, data, format)
         try:
             if self.new_style:
-                return (format or self.format).format(**data)
+                return format.format(**data)
             else:
-                return (format or self.format) % data
+                return format % data
         except (TypeError, KeyError):
-            print('Error while formatting %r' % self.format, file=sys.stderr)
+            print('Error while formatting %r' % format, file=sys.stderr)
             pprint.pprint(data, stream=sys.stderr)
             raise
 
@@ -628,17 +632,18 @@ class Counter(FormatWidgetMixin, WidgetBase):
 class Percentage(FormatWidgetMixin, WidgetBase):
     '''Displays the current percentage as a number with a percent sign.'''
 
-    def __init__(self, format='%(percentage)3d%%', **kwargs):
+    def __init__(self, format='%(percentage)3d%%', na='N/A%%', **kwargs):
+        self.na = na
         FormatWidgetMixin.__init__(self, format=format, **kwargs)
         WidgetBase.__init__(self, format=format, **kwargs)
 
-    def __call__(self, progress, data, format=None):
+    def get_format(self, progress, data, format=None):
         # If percentage is not available, display N/A%
-        if 'percentage' in data and not data['percentage']:
-            return FormatWidgetMixin.__call__(self, progress, data,
-                                              format='N/A%%')
+        percentage = data.get('percentage', base.Undefined)
+        if not percentage and percentage != 0:
+            return self.na
 
-        return FormatWidgetMixin.__call__(self, progress, data)
+        return FormatWidgetMixin.get_format(self, progress, data, format)
 
 
 class SimpleProgress(FormatWidgetMixin, WidgetBase):
@@ -904,6 +909,101 @@ class MultiProgressBar(MultiRangeBar):
         return ranges
 
 
+class GranularMarkers:
+    smooth = ' ▏▎▍▌▋▊▉█'
+    bar = ' ▁▂▃▄▅▆▇█'
+    snake = ' ▖▌▛█'
+    fade_in = ' ░▒▓█'
+    dots = ' ⡀⡄⡆⡇⣇⣧⣷⣿'
+    growing_circles = ' .oO'
+
+
+class GranularBar(AutoWidthWidgetBase):
+    '''A progressbar that can display progress at a sub-character granularity
+    by using multiple marker characters.
+
+    Examples of markers:
+     - Smooth: ` ▏▎▍▌▋▊▉█` (default)
+     - Bar: ` ▁▂▃▄▅▆▇█`
+     - Snake: ` ▖▌▛█`
+     - Fade in: ` ░▒▓█`
+     - Dots: ` ⡀⡄⡆⡇⣇⣧⣷⣿`
+     - Growing circles: ` .oO`
+
+    The markers can be accessed through GranularMarkers. GranularMarkers.dots
+    for example
+    '''
+
+    def __init__(self, markers=GranularMarkers.smooth, left='|', right='|',
+                 **kwargs):
+        '''Creates a customizable progress bar.
+
+        markers - string of characters to use as granular progress markers. The
+                  first character should represent 0% and the last 100%.
+                  Ex: ` .oO`.
+        left - string or callable object to use as a left border
+        right - string or callable object to use as a right border
+        '''
+        self.markers = markers
+        self.left = string_or_lambda(left)
+        self.right = string_or_lambda(right)
+
+        AutoWidthWidgetBase.__init__(self, **kwargs)
+
+    def __call__(self, progress, data, width):
+        left = converters.to_unicode(self.left(progress, data, width))
+        right = converters.to_unicode(self.right(progress, data, width))
+        width -= progress.custom_len(left) + progress.custom_len(right)
+
+        if progress.max_value is not base.UnknownLength \
+                and progress.max_value > 0:
+            percent = progress.value / progress.max_value
+        else:
+            percent = 0
+
+        num_chars = percent * width
+
+        marker = self.markers[-1] * int(num_chars)
+
+        marker_idx = int((num_chars % 1) * (len(self.markers) - 1))
+        if marker_idx:
+            marker += self.markers[marker_idx]
+
+        marker = converters.to_unicode(marker)
+
+        # Make sure we ignore invisible characters when filling
+        width += len(marker) - progress.custom_len(marker)
+        marker = marker.ljust(width, self.markers[0])
+
+        return left + marker + right
+
+
+class FormatLabelBar(FormatLabel, Bar):
+    '''A bar which has a formatted label in the center.'''
+    def __init__(self, format, **kwargs):
+        FormatLabel.__init__(self, format, **kwargs)
+        Bar.__init__(self, **kwargs)
+
+    def __call__(self, progress, data, width, format=None):
+        center = FormatLabel.__call__(self, progress, data, format=format)
+        bar = Bar.__call__(self, progress, data, width)
+
+        # Aligns the center of the label to the center of the bar
+        center_len = progress.custom_len(center)
+        center_left = int((width - center_len) / 2)
+        center_right = center_left + center_len
+        return bar[:center_left] + center + bar[center_right:]
+
+
+class PercentageLabelBar(Percentage, FormatLabelBar):
+    '''A bar which displays the current percentage in the center.'''
+    # %3d adds an extra space that makes it look off-center
+    # %2d keeps the label somewhat consistently in-place
+    def __init__(self, format='%(percentage)2d%%', na='N/A%%', **kwargs):
+        Percentage.__init__(self, format, na=na, **kwargs)
+        FormatLabelBar.__init__(self, format, **kwargs)
+
+
 class Variable(FormatWidgetMixin, VariableMixin, WidgetBase):
     '''Displays a custom variable.'''
 
